# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ubuntu_system_tests.helpers import addressbook
from ubuntu_system_tests.helpers import camera
from ubuntu_system_tests.helpers.camera import fixture_setup as camera_fixture
from ubuntu_system_tests.helpers import dialer_app
from ubuntu_system_tests.helpers import messaging
from ubuntu_system_tests.helpers.processes import is_process_running
from ubuntu_system_tests.helpers import unity8
from ubuntu_system_tests.helpers import wait_until

from ubuntu_system_tests.tests import base


class AppSwitcherTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(camera_fixture.SetCameraAccessRequests())
        self.shell = self.restart_unity().main_window

    def _switch_to_app(self, stage, app_id, app_name):
        """Select an app from the task switcher and check it is displayed."""
        stage.switch_to_app(app_name)
        self.assertTrue(wait_until(
            lambda: self.shell.get_current_focused_app() == app_id))

    def _close_app(self, stage, app_id, app_name):
        """Close an app using task switcher and check process is stopped."""
        self.assertTrue(is_process_running(app_id))
        app_windows = stage.get_app_window_names()
        count = len(app_windows)
        self.assertTrue(app_name in app_windows)
        stage.close_app(app_name)
        app_windows = stage.get_app_window_names()
        self.assertFalse(app_name in app_windows)
        self.assertFalse(is_process_running(app_id))
        self.assertEqual(len(app_windows), count - 1)

    def _launch_applications_and_switcher(self):
        """Launch test applications and open the task switcher
        :return: PhoneStage task switcher object.
        """
        dialer_app.launch_dialer_app_from_launcher()
        messaging.launch_messaging_app_from_launcher()
        addressbook.launch_address_book_from_launcher()
        camera.launch_camera_app_from_launcher()
        stage = self.shell.swipe_to_show_app_switcher()
        stage.swipe_to_top()
        apps = stage.get_app_window_names()
        expected_names = [camera.APP_WIN_NAME,
                          addressbook.APP_WIN_NAME,
                          messaging.APP_WIN_NAME,
                          dialer_app.APP_WIN_NAME,
                          unity8.APP_WIN_NAME]
        self.assertEqual(expected_names, apps)
        return self.shell.swipe_to_show_app_switcher()

    def test_app_selection(self):
        """
        Launch applications and check that it is possible to switch focus
        back to them using the task switcher.
        """
        stage = self._launch_applications_and_switcher()
        self._switch_to_app(stage, dialer_app.APP_WIN_ID,
                            dialer_app.APP_WIN_NAME)
        stage = self.shell.swipe_to_show_app_switcher()
        self._switch_to_app(stage, camera.APP_WIN_ID, camera.APP_WIN_NAME)

    def test_close_app(self):
        """
        Launch applications and check it is possible to close them using
        the swipe actions.
        """
        stage = self._launch_applications_and_switcher()
        self._close_app(stage, dialer_app.APP, dialer_app.APP_WIN_NAME)
